using System;
using System.Collections.Generic;
//using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace TeleportScript
{
    public struct Location
    {
        public GTA.Vector3 position;
        public string Name;
    }

    public class TeleportScript : GTA.Script
    {
        int LocationID = 0;
        List<Location> locations;
        GTA.Blip currentBlip;

        public TeleportScript()
        {
            locations = new List<Location>();

            BindKey(Keys.Add, new GTA.KeyPressDelegate(LocID_Next));
            BindKey(Keys.Subtract, new GTA.KeyPressDelegate(LocID_Previous));
            BindKey(Keys.Multiply, new GTA.KeyPressDelegate(Loc_Teleport));
            BindKey(Keys.Oemplus, new GTA.KeyPressDelegate(Loc_Add));
            BindKey(Keys.OemOpenBrackets, new GTA.KeyPressDelegate(Load));
            BindKey(Keys.OemCloseBrackets, new GTA.KeyPressDelegate(Save));
            BindConsoleCommand("tele_add", new GTA.ConsoleCommandDelegate(CMD_Add), this);
        }

        public void CMD_Add(string[] args)
        {
            if (args.Length < 2)
            {
                GTA.Console.WriteLine("Usage: tele_add name");
                return;
            }

            Location location = new Location();
            location.position = PlayerObject.Character.Position;
            location.Name = args[1];

            locations.Add(location);

            GTA.Console.WriteLine("Location added!");
        }

        public void LocID_Next() {
            if (currentBlip != null)
            {
                currentBlip.Delete();
            }

            LocationID++;

            if (LocationID > (locations.Count) - 1)
            {
                LocationID = 0;
            }

            currentBlip = GTA.Blip.AddBlip(locations[LocationID].position, GTA.BlipIcon.Misc_RedFlag);

            GTA.Game.DisplayTextBox(locations[LocationID].Name);
        }

        public void LocID_Previous()
        {
            if (currentBlip != null)
            {
                currentBlip.Delete();
            }

            LocationID--;

            if (LocationID < 0)
            {
                LocationID = locations.Count - 1;
            }

            currentBlip = GTA.Blip.AddBlip(locations[LocationID].position, GTA.BlipIcon.Misc_RedFlag);

            GTA.Game.DisplayTextBox(locations[LocationID].Name);
        }

        public void Loc_Teleport()
        {
            PlayerObject.Character.Position = locations[LocationID].position;

            if (currentBlip != null)
            {
                currentBlip.Delete();
            }

            GTA.Game.DisplayTextBox("Teleported to " + locations[LocationID].Name);
        }

        public void Loc_Add()
        {
            Location location = new Location();
            location.position = PlayerObject.Character.Position;
            location.Name = "Unnamed location";

            locations.Add(location);

            GTA.Game.DisplayTextBox("Location added to list.");
        }

        public void Load()
        {
            if (!System.IO.File.Exists(GTA.Game.InstallFolder + "\\scripts\\datafiles\\teleports.txt"))
            {
                GTA.Game.DisplayTextBox("The location storage file does not exist.");
                return;
            }

            System.IO.StreamReader sr = new System.IO.StreamReader(GTA.Game.InstallFolder + "\\scripts\\datafiles\\teleports.txt");

            locations.Clear();

            while (!sr.EndOfStream)
            {
                string line = sr.ReadLine();

                System.Text.RegularExpressions.Regex re = new System.Text.RegularExpressions.Regex("(.+?) (.+?) (.+?) -- (.+)");
                System.Text.RegularExpressions.Match match = re.Match(line);

                if (match.Success)
                {
                    Location loc = new Location();
                    loc.position = new GTA.Vector3();
                    loc.position.pX = float.Parse(match.Groups[1].Value);
                    loc.position.pY = float.Parse(match.Groups[2].Value);
                    loc.position.pZ = float.Parse(match.Groups[3].Value);

                    loc.Name = match.Groups[4].Value;

                    locations.Add(loc);
                }
            }

            sr.Close();

            if (currentBlip != null)
            {
                currentBlip.Delete();
            }

            GTA.Game.DisplayTextBox(string.Format("Loaded {0} locations.", locations.Count));
        }

        public void Save()
        {
            System.IO.StreamWriter sw = new System.IO.StreamWriter(GTA.Game.InstallFolder + "\\scripts\\datafiles\\teleports.txt");
            foreach (Location location in locations)
            {
                sw.WriteLine(string.Format("{0} {1} {2} -- {3}", location.position.pX, location.position.pY, location.position.pZ, location.Name));
            }

            sw.Close();

            if (currentBlip != null)
            {
                currentBlip.Delete();
            }

            GTA.Game.DisplayTextBox(string.Format("Saved {0} locations.", locations.Count));
        }
    }
}
